@@ -208,12 +208,54 @@ if ($ipv4['dx_properties']['enabled'])
 		$range = $addr_ranges[$input['addr_range']]['arange4'];
 		$rs.= $iptables." -t nat -A POSTROUTING -s ".$range['ip4']."/".$range['mask']." -o ".$output['if']." -j MASQUERADE\n";
 	}
+
+
+	$rs.= "# IPv4--pmserv :\n\n";
+	$pmserv = $ipv4['portmapped_services'][0];
+	if ($pmserv['dx_properties']['enabled'])
+	{
+		$forward_in_rule_pm = "";
+		$rs.= $iptables." -t nat -N port_mapping\n";
+		$s = get_infos($pmserv['pservice'], array('proto', 'outside_port', 'if', 'port'));
+		$i = 0;
+		foreach ($s as $key => $val)
+		{
+			if ($pmserv['pservice'][$i]['dx_properties']['enabled'])
+			{
+				if (($val['proto'] == "tcp/udp") || ($val['proto'] == "udp/tcp"))
+					$plist = array("tcp", "udp");
+				else
+					$plist = array($val['proto']);
+				$host = $g_ifs[$val['if']]['ip4'];
+
+				for ($j=0; $j<count($plist); $j++)
+				{
+					if ($val['outside_port'])
+						$rs.= $iptables." -t nat -A port_mapping -p ".$plist[$j]." --dport ".$val['outside_port']." -j DNAT --to-destination ".$host.":".$val['port']."\t# ".$key."\n";
+					else
+						$rs.= $iptables." -t nat -A port_mapping -p ".$plist[$j]." --dport ".$val['port']." -j DNAT --to-destination ".$host."\t\t# ".$key."\n";
+					$forward_in_rule_pm.= $iptables." -t filter -A forward_in_packets_allowed -p ".$plist[$j]." -d ".$host." --dport ".$val['port']." -m state --state NEW,ESTABLISHED,RELATED -j ACCEPT\n";
+				}
+			}
+			else
+				$rs.= "#       remote service '".$key."' disabled\n";
+			$i++;
+		}
+		$rs.= $iptables." -t nat -A PREROUTING -i ".$output['if']." -j port_mapping\n";
+	}
+	else
+	{
+		$rs.= "#    disabled\n";
+	}
+	$rs.= "\n";
+
 	if ($forwarding['dx_properties']['state_filtering'])
 	{
 		$rs.= create_rule('forwarding', "[EMIR] Bad fwding state : ");
 		
 		$forward_in_rule = "forward_in_packets_allowed";
 		$rs.= $iptables." -t filter -N forward_in_packets_allowed\n";
+		$rs.= $forward_in_rule_pm;
 		$rs.= $iptables." -t filter -A forward_in_packets_allowed -m state --state ESTABLISHED,RELATED -j ACCEPT\n";
 		$rs.= $iptables." -t filter -A forward_in_packets_allowed -j ".rule('forwarding', $forwarding['dx_properties']['bad'])."\n";
 		$forward_out_rule = "forward_out_packets_allowed";
@@ -449,44 +491,6 @@ if ($ipv4['dx_properties']['enabled'])
 	$rs.= "\n";
 
 
-	$rs.= "# IPv4--pmserv :\n\n";
-	$pmserv = $ipv4['portmapped_services'][0];
-	if ($pmserv['dx_properties']['enabled'])
-	{
-		$rs.= $iptables." -t nat -N port_mapping\n";
-		$s = get_infos($pmserv['pservice'], array('proto', 'outside_port', 'if', 'port'));
-		$i = 0;
-		foreach ($s as $key => $val)
-		{
-			if ($pmserv['pservice'][$i]['dx_properties']['enabled'])
-			{
-				if (($val['proto'] == "tcp/udp") || ($val['proto'] == "udp/tcp"))
-					$plist = array("tcp", "udp");
-				else
-					$plist = array($val['proto']);
-				$host = $g_ifs[$val['if']]['ip4'];
-
-				for ($j=0; $j<count($plist); $j++)
-				{
-					if ($val['outside_port'])
-						$rs.= $iptables." -t nat -A port_mapping -p ".$plist[$j]." --dport ".$val['outside_port']." -j DNAT --to-destination ".$host.":".$val['port']."\t# ".$key."\n";
-					else
-						$rs.= $iptables." -t nat -A port_mapping -p ".$plist[$j]." --dport ".$val['port']." -j DNAT --to-destination ".$host."\t\t# ".$key."\n";
-				}
-			}
-			else
-				$rs.= "#       remote service '".$key."' disabled\n";
-			$i++;
-		}
-		$rs.= $iptables." -t nat -A PREROUTING -i ".$output['if']." -j port_mapping\n";
-	}
-	else
-	{
-		$rs.= "#    disabled\n";
-	}
-	$rs.= "\n";
-
-
 	$rs.= "# IPv4--security :\n\n";
 	if ($security['dx_properties']['enabled'])
 	{
@@ -1204,4 +1208,32 @@ function rule($key, $type, $ipv4=true)
 	}
 }
 
+/* futur developpement
+
+function is_ip_in_range($ip, $net, $mask)
+{
+	$lnet = ip2long($net);
+	$lip = ip2long($ip);
+
+	$binnet = str_pad(decbin($lnet), 32, "0", "STR_PAD_LEFT");
+	$firstpart = substr($binnet, 0, $mask);
+	$binip = str_pad(decbin($lip), 32, "0", "STR_PAD_LEFT");
+	$firstip = substr($binip ,0 ,$mask);
+
+	return (strcmp($firstpart, $firstip) == 0);
+}
+
+function get_subnet($ip, $mask)
+{
+	$lip = ip2long($ip);
+
+	$binip = str_pad(decbin($lip), 32, "0", "STR_PAD_LEFT");
+	$firstip = substr($binip ,0 ,$mask);
+	$subnet = str_pad($firstip, 32, "0", "STR_PAD_RIGHT");
+
+	return long2ip(bindec($subnet));
+}
+
+*/
+
 ?>
